<?php
namespace Fiscal\Logic\NotaFiscal\Saida;
use \Fiscal\Entities\Notafiscal;
use \Geral\Entities\Tipooperac;
use \Indeterminado\Entities\Validanftpop;
use \Fiscal\Logic\Impostos\ValidacaoImpostos;
use \Fiscal\Util\Parametros;
use \Fiscal\Logic\NotaFiscal\Exceptions\NFValidaSaidaException;
use Fiscal\Logic\NotaFiscal\Exceptions\NFSaidaException;
use \Geral\Entities\Itemproduto;
/**
 * Created by JetBrains PhpStorm.
 * User: gustavoavila
 * Date: 17/01/13
 * Time: 15:15
 * To change this template use File | Settings | File Templates.
 */
class NFValida
{
    
    /**@var $notafiscal Notafiscal*/
    public $notafiscal;
    
    public function __construct(Notafiscal $notafiscal)
    {
        $this->notafiscal = $notafiscal;
    }
    
    /**
     * Valida se nota fiscal ja foi emitida.
     * @param \Fiscal\Entities\
     * @return bool
     * @throws \Fiscal\Logic\NotaFiscal\Exceptions\NFValidaSaidaException
     */
    public function validaEmissao()
    {
        $params = array($this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf());
        $obj = Notafiscal::getObjectByWhere("CDFILIAL = ? AND NRLANCTONF = ? AND IDSTATUS IN ('A', 'E')", $params);
        if ($obj instanceof \Fiscal\Entities\Notafiscal) {
            throw new NFValidaSaidaException("Operação Bloqueada! Nota Fiscal emitida.");
        }

        return true;
    }

    /**
     * Valida se data foi informada nos vencimentos de cobranca
     * @param \Fiscal\Entities\
     * @return bool
     * @throws \Fiscal\Logic\NotaFiscal\Exceptions\NFValidaSaidaException
     */
    public function validaVencimentosCobranca()
    {
        $arContaRec = $this->notafiscal->getContasReceber();
        /**@var $contaRec \Financeiro\Entities\Contareceber*/
        foreach($arContaRec as $contaRec) {
            if ($contaRec->getDtorigvenrec() == null) {
                throw new NFValidaSaidaException("Vencimento deve ser informado.");
            }
        }

        return true;
    }

    /**
     * Valida total calculado da notafiscal com o valor dos vencimentos.
     * @param \Fiscal\Entities\
     * @param $totalNF
     * @return bool
     * @throws \Fiscal\Logic\NotaFiscal\Exceptions\NFValidaSaidaException
     */
    public function validaTotalNF($totalNF)
    {
        $diffValor = round($totalNF, 2) - round($this->notafiscal->somaContasReceber(), 2);
        if ($diffValor !== 0) {
            $idorigdestnf = $this->notafiscal->getIdorigdestnf();
            if (empty($idorigdestnf) || $idorigdestnf === 'C' || $idorigdestnf === 'U') {
                throw new NFValidaSaidaException("A soma dos vencimentos difere do valor total da NF. Diferença: {$diffValor}");
            }
        }

        return true;
    }

    /**
     * Valida inscricao estadual.
     * Caso esteje parametrizado para ser igual valida se o destino informado possui inscrição estadual igual, caso não tenha gera exceção.
     * Caso esteje parametrizado para ser diferente valida se o destino informado possui inscrição estadual diferente, caso não tenha gera exceção.
     * @param \Fiscal\Entities\
     * @throws \Fiscal\Logic\NotaFiscal\Exceptions\NFValidaSaidaException
     */
    public function validaInscricaoEstadual()
    {
        try{
            $validnftpop = $this->notafiscal->getValidanftpop();
            if ( $validnftpop->getIdinscestaigu() === Validanftpop::UTILIZA_INSCRICAO_ESTADUAL_IGUAL
                 && $validnftpop->getIdinscestadif() === Validanftpop::NAO_UTILIZA_INSCRICAO_ESTADUAL_DIFERENTE) {

                $this->validaInscricaoEstadualIgual($this->notafiscal);

            } elseif ($validnftpop->getIdinscestaigu() === Validanftpop::NAO_UTILIZA_INSCRICAO_ESTADUAL_IGUAL
                             && $validnftpop->getIdinscestadif() === Validanftpop::UTILIZA_INSCRICAO_ESTADUAL_DIFERENTE){

                $this->validaInscricaoEstadualDiferente($this->notafiscal);

            }
        } catch(NFValidaSaidaException $e) {
            throw $e;
        }

    }

    /**
     * Valida se o destino informado possui inscrição estadual igual, caso não tenha gera exceção.
     * @param \Fiscal\Entities\
     * @throws \Fiscal\Logic\NotaFiscal\Exceptions\NFValidaSaidaException
     */
    public function validaInscricaoEstadualIgual()
    {
        $origDest = $this->notafiscal->getIdorigdestnf();
        switch ($origDest) {
            case Notafiscal::ID_ORIGEM_DESTINO_FILIAL :
                $filial = $this->notafiscal->getFilirece();
                if ($this->notafiscal->getCdinscesta() !== $filial->getCdinscesta()) {
                    throw new NFValidaSaidaException("Inscrição estadual do emitente diferente da inscrição estadual do destinatário/remetente.
                                                      Inscrição estadual deve ser igual para este tipo de operação.");
                }
                break;
            case Notafiscal::ID_ORIGEM_DESTINO_FORNECEDOR :
                $fornecedor = $this->notafiscal->getForneced();
                if ($this->notafiscal->getCdinscesta() !== $fornecedor->getNrinsestforn()) {
                    throw new NFValidaSaidaException("Inscrição estadual do emitente diferente da inscrição estadual do destinatário/remetente.
                                                      Inscrição estadual deve ser igual para este tipo de operação.");
                }
                break;
            case Notafiscal::ID_ORIGEM_DESTINO_CLIENTE :
                $cliente = $this->notafiscal->getCliente();
                if ($this->notafiscal->getCdinscesta() !== $cliente->getNrinsestclie()) {
                    throw new NFValidaSaidaException("Inscrição estadual do emitente diferente da inscrição estadual do destinatário/remetente.
                                                      Inscrição estadual deve ser igual para este tipo de operação.");
                }
                break;
        }
    }

    /**
     * Caso esteje parametrizado para ser diferente valida se o destino informado possui inscrição estadual diferente, caso não tenha gera exceção.
     * @param \Fiscal\Entities\
     * @throws \Fiscal\Logic\NotaFiscal\Exceptions\NFValidaSaidaException
     */
    public function validaInscricaoEstadualDiferente()
    {
        $origDest = $this->notafiscal->getIdorigdestnf();
        switch ($origDest) {
            case Notafiscal::ID_ORIGEM_DESTINO_FILIAL :
                $filial = $this->notafiscal->getFilirece();
                if ($this->notafiscal->getCdinscesta() == $filial->getCdinscesta()) {
                    throw new NFValidaSaidaException("Inscrição estadual do emitente igual a inscrição estadual do destinatário/remetente.
                                                      Inscrição estadual deve ser diferente para este tipo de operação");
                }
                break;
            case Notafiscal::ID_ORIGEM_DESTINO_FORNECEDOR :
                $fornecedor = $this->notafiscal->getForneced();
                if ($this->notafiscal->getCdinscesta() == $fornecedor->getNrinsestforn()) {
                    throw new NFValidaSaidaException("Inscrição estadual do emitente igual a inscrição estadual do destinatário/remetente.
                                                      Inscrição estadual deve ser diferente para este tipo de operação");
                }
                break;
            case Notafiscal::ID_ORIGEM_DESTINO_CLIENTE :
                $cliente = $this->notafiscal->getCliente();
                if ($this->notafiscal->getCdinscesta() == $cliente->getNrinsestclie()) {
                    throw new NFValidaSaidaException("Inscrição estadual do emitente igual a inscrição estadual do destinatário/remetente.
                                                      Inscrição estadual deve ser diferente para este tipo de operação");
                }
                break;
        }
    }

    /**
     * Função para verificar os impostos obrigatórios por tipo de operação caso a filial utilize Controle Fiscal
     * @return bool
     * @throws \Exception
     */
    public function validaImpostosObrigatorios()
    {
        try {
            $validaClass = new ValidacaoImpostos();
            $validaClass->validaImpostosObrigatorios($this->notafiscal);
        } catch(\Exception $e) {
            throw $e;
        }

        return true;
    }

    public function deletaNota(){
        \Fiscal\Logic\NotaFiscal\ProceduresNF::procedureDeletaNF($this->notafiscal);
    }
    
    public function validaQtItemDevolucaoAoInserir($quantidade_antiga, $quantidade, $quantidade_resta_devolver, $qtJaDevolvida){
    	if($quantidade_antiga == 0 && $quantidade > $quantidade_resta_devolver){
    		return NFSaidaException::qtItemDevolverMenorIgualQtOriginal($qtJaDevolvida);
    	}
    	return true;
    }

    public function validaQtItemDevolucaoAoAlterar($lancamento, $filialOrigem, $lancamentoOrigem, $nrdecitotanf, $cdarvprod, $quantidade, $quantidade_origem, $quantidade_antiga, $quantidade_origem, $qtJaDevolvida, $produto){
    	$qtitemdevo = Itemproduto::getRepository()->consultaQuantidadeJaDevolvida($lancamento, $filialOrigem, $lancamentoOrigem, $nrdecitotanf, $produto, $cdarvprod);
    	
    	if($qtitemdevo == 0 && $quantidade > $quantidade_origem){
    		return NFSaidaException::qtItemDevolverMenorIgualQtOrigem();
    	}elseif($quantidade + $qtitemdevo > $quantidade_origem){
    		return NFSaidaException::qtItemDevolverMenorIgualQtOriginal($qtJaDevolvida);
    	}
    	return true;
    }
}
